import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { SvgNmrAlignBottom, SvgNmrAlignCenter, SvgNmrApodization, SvgNmrBaselineCorrection, SvgNmrFourierTransform, SvgNmrIntegrate, SvgNmrMultipleAnalysis, SvgNmrOverlay3, SvgNmrOverlay3Aligned, SvgNmrPeakPicking, SvgNmrPhaseCorrection, SvgNmrRangePicking, SvgNmrRealImag, SvgNmrZeroFilling, } from 'cheminfo-font';
import { useCallback } from 'react';
import { FaSearchPlus, FaExpand, FaDiceFour, FaFileImport, FaFileExport, } from 'react-icons/fa';
import { PiKnifeBold } from 'react-icons/pi';
import { Toolbar } from 'react-science/ui';
import { useChartData } from '../context/ChartContext';
import { useDispatch } from '../context/DispatchContext';
import { useLoader } from '../context/LoaderContext';
import { usePreferences } from '../context/PreferencesContext';
import { CustomToolTip } from '../elements/CustomToolTip';
import { ToolbarPopoverItem, } from '../elements/ToolbarPopoverItem';
import useCheckExperimentalFeature from '../hooks/useCheckExperimentalFeature';
import { useCheckToolsVisibility, } from '../hooks/useCheckToolsVisibility';
import useDatumWithSpectraStatistics from '../hooks/useDatumWithSpectraStatistics';
import { useDialogToggle } from '../hooks/useDialogToggle';
import useExport from '../hooks/useExport';
import useToolsFunctions from '../hooks/useToolsFunctions';
import { useVerticalAlign } from '../hooks/useVerticalAlign';
import { ImportPublicationStringModal } from '../modal/ImportPublicationStringModal';
import { LoadJCAMPModal } from '../modal/LoadJCAMPModal';
import SaveAsModal from '../modal/SaveAsModal';
import { MetaImportationModal } from '../modal/metaImportation/MetaImportationModal';
import { options } from './ToolTypes';
import { EXPORT_MENU, IMPORT_MENU } from './toolbarMenu';
function isPopoverToolItem(item) {
    return !!item?.menuItems;
}
export default function ToolBar() {
    const { toolOptions: { selectedTool }, } = useChartData();
    const isButtonVisible = useCheckToolsVisibility();
    const dispatch = useDispatch();
    const { isRealSpectrumShown, changeSpectrumViewHandler, changeDisplayViewModeHandler, alignSpectraVerticallyHandler, handleChangeOption, handleFullZoomOut, } = useToolsFunctions();
    const handleChange = useCallback((selectedOption) => {
        handleChangeOption(selectedOption);
    }, [handleChangeOption]);
    const openLoader = useLoader();
    const isExperimentalFeature = useCheckExperimentalFeature();
    const { dialog, closeDialog, openDialog } = useDialogToggle({
        loadJCAMP: false,
        importPublicationString: false,
        metaImportation: false,
        saveAs: false,
    });
    const verticalAlign = useVerticalAlign();
    const { fidCounter, ftCounter } = useDatumWithSpectraStatistics();
    const importMenu = isExperimentalFeature
        ? IMPORT_MENU
        : IMPORT_MENU.filter((item) => item.id !== 'importPublicationString');
    const exportMenu = isExperimentalFeature
        ? EXPORT_MENU
        : EXPORT_MENU.filter((item) => item.id !== 'nmre');
    const { current: { general: { invert }, }, } = usePreferences();
    const handleOnFFTFilter = useCallback(() => {
        dispatch({
            type: 'APPLY_FFT_FILTER',
        });
    }, [dispatch]);
    const handleFFtDimension1Filter = useCallback(() => {
        dispatch({
            type: 'APPLY_FFT_DIMENSION_1_FILTER',
        });
    }, [dispatch]);
    const handleFFtDimension2Filter = useCallback(() => {
        dispatch({
            type: 'APPLY_FFT_DIMENSION_2_FILTER',
        });
    }, [dispatch]);
    const { saveAsSVGHandler, saveAsPNGHandler, saveAsJSONHandler, saveToClipboardHandler, } = useExport();
    function importHandler(data) {
        switch (data?.id) {
            case 'importFile':
                openLoader();
                break;
            case 'importJDX':
                openDialog('loadJCAMP');
                break;
            case 'importPublicationString':
                openDialog('importPublicationString');
                break;
            case 'importMetaInformation':
                openDialog('metaImportation');
                break;
            default:
        }
    }
    function exportHandler(data) {
        switch (data?.id) {
            case 'svg':
                void saveAsSVGHandler();
                break;
            case 'png':
                void saveAsPNGHandler();
                break;
            case 'json':
                saveAsJSONHandler();
                break;
            case 'advance_save':
                openDialog('saveAs');
                break;
            case 'copy':
                void saveToClipboardHandler();
                break;
            default:
                break;
        }
    }
    const toolItems = [
        {
            id: 'zoom',
            tooltip: {
                title: options.zoom.label,
                subTitles: [
                    { title: 'Vertical', shortcuts: ['Scroll wheel'] },
                    { title: 'Horizontal', shortcuts: ['SHIFT', 'Scroll wheel'] },
                    { title: 'Pan', shortcuts: ['Right mouse'] },
                ],
            },
            icon: _jsx(FaSearchPlus, {}),
        },
        {
            id: 'zoomOut',
            tooltip: {
                title: 'Zoom out',
                subTitles: [
                    { title: 'Horizontal', shortcuts: ['f'] },
                    { title: 'Horizontal and Vertical', shortcuts: ['f', 'f'] },
                ],
                description: 'Zoom out by double-clicking the left mouse button, and fully zoom out horizontally by pressing the key "f". Alternatively, press the key "ff" to fit the spectra horizontally and vertically.',
                link: 'https://docs.nmrium.org/help/zoom-and-scale',
            },
            onClick: handleFullZoomOut,
            icon: _jsx(FaExpand, {}),
        },
        {
            id: 'peakPicking',
            tooltip: {
                title: options.peakPicking.label,
                shortcuts: ['p'],
                description: 'Detect peaks manually or automatically in the spectrum.',
                link: 'https://docs.nmrium.org/20_1d-spectra/peaks/peaks',
            },
            icon: _jsx(SvgNmrPeakPicking, {}),
        },
        {
            id: 'integral',
            tooltip: {
                title: options.integral.label,
                shortcuts: ['i'],
                description: `Manually integrate the spectrum. Click, drag, and release ${!invert ? 'while holding SHIFT' : ''} to draw the integral. Resize the integrals by moving the edges. Cut an integral with ${!invert ? 'SHIFT +' : ''} click.`,
                link: 'https://docs.nmrium.org/help/integrations',
            },
            icon: _jsx(SvgNmrIntegrate, {}),
        },
        {
            id: 'zonePicking',
            tooltip: {
                title: options.zonePicking.label,
                shortcuts: ['r'],
                description: `Draw 2D zones by clicking, dragging, and releasing${!invert ? 'while holding SHIFT' : ''}. Alternatively, detect zones automatically.`,
                link: 'https://docs.nmrium.org/30_2d-spectra/zones/zones',
            },
            icon: _jsx(FaDiceFour, {}),
        },
        {
            id: 'slicing',
            tooltip: {
                title: options.slicing.label,
                description: 'Display the horizontal and vertical slices of the selected 2D spectrum at the level of the pointer.',
                link: 'https://docs.nmrium.org/30_2d-spectra/slicing/slicing',
            },
            icon: _jsx(PiKnifeBold, {}),
        },
        {
            id: 'rangePicking',
            tooltip: {
                title: options.rangePicking.label,
                shortcuts: ['r'],
                description: `Define ranges and analyse multiplet automatically or manually.  Click, drag, and release ${!invert ? 'while holding SHIFT' : ''} to draw the range. Ranges can be resized by moving the edges.`,
                link: 'https://docs.nmrium.org/20_1d-spectra/ranges/ranges',
            },
            icon: _jsx(SvgNmrRangePicking, {}),
        },
        {
            id: 'multipleSpectraAnalysis',
            tooltip: options.multipleSpectraAnalysis.label,
            icon: _jsx(SvgNmrMultipleAnalysis, {}),
            checkOptions: { checkSpectrumType: false },
            condition: ftCounter > 0,
        },
        {
            id: 'apodization',
            tooltip: {
                title: options.apodization.label,
                shortcuts: ['a'],
                description: 'Apply mathematical function that the FID is multiplied by before Fourier Transform.',
            },
            icon: _jsx(SvgNmrApodization, {}),
        },
        {
            id: 'zeroFilling',
            tooltip: {
                title: options.zeroFilling.label,
                shortcuts: ['z'],
                description: 'Improve spectrum quality by increasing the number of points. By default, the number of points is twice as many as in the original FID.',
                link: 'https://docs.nmrium.org/20_1d-spectra/preprocessing/preprocessing#zero-filling',
            },
            icon: _jsx(SvgNmrZeroFilling, {}),
        },
        {
            id: 'phaseCorrection',
            tooltip: {
                title: options.phaseCorrection.label,
                shortcuts: ['a'],
                description: `Correct the spectrum phase manually or automatically. For manual phase correction, define the pivot${!invert ? ' using SHIFT +' : ''} click, then press the PH0 and PH1 button, and move the mouse horizontally.`,
                link: 'https://docs.nmrium.org/20_1d-spectra/phase/phase',
            },
            icon: _jsx(SvgNmrPhaseCorrection, {}),
        },
        {
            id: 'phaseCorrectionTwoDimensions',
            tooltip: {
                title: options.phaseCorrectionTwoDimensions.label,
                shortcuts: ['a'],
                description: 'To phase the spectrum after the FFT.',
            },
            icon: _jsx(SvgNmrPhaseCorrection, {}),
        },
        {
            id: 'baselineCorrection',
            tooltip: {
                title: options.baselineCorrection.label,
                shortcuts: ['b'],
                description: `Correct the baseline of the spectrum. You should draw zones corresponding to the noise using click, drag, release${!invert ? 'while holding SHIFT' : ''}.`,
                link: 'https://docs.nmrium.org/20_1d-spectra/baseline/baseline',
            },
            icon: _jsx(SvgNmrBaselineCorrection, {}),
        },
        {
            id: 'exclusionZones',
            tooltip: {
                title: options.exclusionZones.label,
                shortcuts: ['e'],
                description: `Define exclusion zones by clicking, dragging, releasing${!invert ? ' while holding SHIFT' : ''}. This option is practical for excluding large peaks like solvents.`,
            },
            icon: _jsx(SvgNmrMultipleAnalysis, {}),
            checkOptions: { checkSpectrumType: false },
            condition: ftCounter > 0,
        },
        {
            id: 'fft',
            tooltip: {
                title: options.fft.label,
                shortcuts: ['t'],
                description: 'Perform a Fourier transform on the FID.',
                link: 'https://docs.nmrium.org/20_1d-spectra/ft/ft/',
            },
            onClick: handleOnFFTFilter,
            icon: _jsx(SvgNmrFourierTransform, {}),
        },
        {
            id: 'fftDimension1',
            tooltip: options.fftDimension1.label,
            onClick: handleFFtDimension1Filter,
            icon: _jsx(SvgNmrFourierTransform, {}),
        },
        {
            id: 'fftDimension2',
            tooltip: options.fftDimension2.label,
            onClick: handleFFtDimension2Filter,
            icon: _jsx(SvgNmrFourierTransform, {}),
        },
        {
            id: 'import',
            tooltip: options.import.label,
            icon: _jsx(FaFileImport, {}),
            menuItems: importMenu,
            onClick: importHandler,
        },
        {
            id: 'exportAs',
            tooltip: options.exportAs.label,
            icon: _jsx(FaFileExport, {}),
            menuItems: exportMenu,
            onClick: exportHandler,
        },
        {
            id: 'spectraStackAlignments',
            tooltip: {
                title: options.spectraStackAlignments.label,
                shortcuts: ['s'],
                description: 'Toggle between stack mode and all the spectra aligned at the bottom.',
            },
            icon: verticalAlign === 'stack' ? (_jsx(SvgNmrOverlay3Aligned, {})) : (_jsx(SvgNmrOverlay3, {})),
            condition: ftCounter > 1,
            onClick: changeDisplayViewModeHandler,
        },
        {
            id: 'realImaginary',
            tooltip: {
                title: `Display ${isRealSpectrumShown ? 'real' : 'imaginary'}`,
                description: 'Toggle between the imaginary and real parts of the spectrum.',
                link: 'https://docs.nmrium.org/20_1d-spectra/views/views',
            },
            icon: _jsx(SvgNmrRealImag, {}),
            onClick: changeSpectrumViewHandler,
        },
        {
            id: 'spectraCenterAlignments',
            tooltip: {
                title: `Baseline ${verticalAlign === 'bottom' ? 'center' : 'bottom'}`,
                shortcuts: ['c'],
                description: 'Toggle between alignment of the spectra in the center or at the bottom of the display.',
            },
            icon: verticalAlign === 'bottom' ? (_jsx(SvgNmrAlignCenter, {})) : (_jsx(SvgNmrAlignBottom, {})),
            condition: ftCounter > 0 || fidCounter > 0,
            onClick: alignSpectraVerticallyHandler,
        },
    ];
    return (_jsxs(_Fragment, { children: [_jsx(LoadJCAMPModal, { isOpen: dialog.loadJCAMP, onCloseDialog: closeDialog }), _jsx(ImportPublicationStringModal, { isOpen: dialog.importPublicationString, onClose: closeDialog }), _jsx(MetaImportationModal, { isOpen: dialog.metaImportation, onCloseDialog: closeDialog }), _jsx(SaveAsModal, { isOpen: dialog.saveAs, onCloseDialog: closeDialog }), _jsx(Toolbar, { vertical: true, children: toolItems.map((item) => {
                    const { id, icon, tooltip, checkOptions, condition } = item;
                    const show = isButtonVisible(id, checkOptions) &&
                        (condition === undefined || condition);
                    if (!show)
                        return null;
                    if (isPopoverToolItem(item)) {
                        const { menuItems, onClick } = item;
                        return (_jsx(ToolbarPopoverItem, { options: menuItems, tooltip: typeof tooltip === 'string' ? (tooltip) : (_jsx(CustomToolTip, { ...tooltip })), tooltipProps: {
                                minimal: true,
                            }, id: id, active: selectedTool === id, icon: icon, onClick: onClick }, id));
                    }
                    const { onClick } = item;
                    return (_jsx(Toolbar.Item, { onClick: onClick || (() => handleChange(id)), tooltip: typeof tooltip === 'string' ? (tooltip) : (_jsx(CustomToolTip, { ...tooltip })), tooltipProps: {
                            minimal: true,
                        }, id: id, active: selectedTool === id, icon: icon }, id));
                }) })] }));
}
